@@ -78,24 +78,26 @@ operation slow rootBad yes no = f [] (step_ yes) (step_ no)
 
         -- parts is a series of path components joined with trailing / characters
         f parts yes no
-            | StepEverything <- stepNext no = return []
-            | not slow, StepOnly xs <- stepNext yes = g parts yes no xs False
+            | StepEverything <- stepNext no = pure []
+            | not slow, StepOnly xs <- stepNext yes = g parts yes no xs
             | otherwise = do
                 xs <- filter (not . all (== '.')) <$> getDirectoryContents (root ++ parts)
-                g parts yes no xs True
+                g parts yes no xs
 
-        -- doesExist means that one of doesFileExist or doesDirectoryExist is true
-        g parts yes no xs doesExist =
+        g parts yes no xs =
             concatForM (sort xs) $ \x -> do
                 let path = root ++ parts ++ x
                 -- deliberately shadow since using yes/no from now on would be wrong
-                yes <- return $ stepApply yes x
-                no <- return $ stepApply no x
-                isFile <- if stepDone yes /= [] && stepDone no == [] then Just <$> doesFileExist path else return Nothing
+                yes <- pure $ stepApply yes x
+                no <- pure $ stepApply no x
+                isFile <- whenMaybe (stepDone yes /= [] && stepDone no == []) (doesFileExist path)
                 case isFile of
-                    Just True -> return [parts ++ x]
-                    _ | StepEverything <- stepNext no -> return []
-                      | StepOnly [] <- stepNext yes -> return []
+                    Just True -> pure [parts ++ x]
+                    _ | StepEverything <- stepNext no -> pure []
+                      | StepOnly [] <- stepNext yes -> pure []
                       | otherwise -> do
-                        b <- if doesExist && isFile == Just False then return True else doesDirectoryExist path
-                        if not b then return [] else f (parts ++ x ++ "/") yes no
+                        -- Here we used to assume that getDirectoryContents means something exists,
+                        -- doesFileExists is False, therefore this must be a directory.
+                        -- That's not true in the presence of symlinks.
+                        b <- doesDirectoryExist path
+                        if not b then pure [] else f (parts ++ x ++ "/") yes no
